package de.ids_mannheim.korap.openthesaurus;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class ThesaurusDictionaryTest {

    String newLine = System.getProperty("line.separator");

    @Test
    @DisplayName("Regex applied to text fields should remove text in parentheses and strip whitespaces")
    void cleanStringTest() {
        String[][] regexStrings = new String[][] {
                {"(some annot.)(some other) Word1 Word2 Word3    (annot.)(other)", "Word1 Word2 Word3"},
                {"mit Füßen treten (ein Recht) (ugs., fig.)", "mit Füßen treten"},
                {"Atomspaltung", "Atomspaltung"},
                //{" Word ","Word"} // doesn't pass yet!
        };
        for (String[] pair : regexStrings) {
            String source = pair[0];
            String target = pair[1];
            assertEquals(target, ThesaurusDictionary.cleanString(source));
        }
    }

    @Test
    @DisplayName("Comments and legal notice should be filtered out from stream")
    void getAndFilterLinesTest() throws IOException {
        String target = "Atomspaltung;Kernspaltung (fachspr., technisch);Fission;Kernfission";
        String source = String.join(newLine,
                "# OpenThesaurus - German Thesaurus in text format",
                target,
                "=============================================================",
                "**** Creative Commons Namensnennung-Share Alike 4.0 International Public License ****",
                "Durch die Ausübung der lizenzierten Rechte (wie unten definiert) erklären Sie..."
        );

        String result = ThesaurusDictionary.getAndFilterLines(
                new ByteArrayInputStream(source.getBytes(StandardCharsets.UTF_8))
        ).collect(Collectors.joining());

        assertEquals(result, target);
    }


    <K, V extends List> boolean twoMapsAreEqual(Map<K, V> map1, Map<K, V> map2) {
        if (map1.size() != map2.size()) return false;
        for (Map.Entry<K, V> pair : map1.entrySet()) {
            K keyTarget = pair.getKey();
            V valuesTarget = pair.getValue();
            if (!map2.containsKey(keyTarget)) return false;
            else {
                V valuesCompare = map2.get(keyTarget);
                if (!valuesCompare.containsAll(valuesTarget)) return false;
            }
        }
        return true;
    }

    @Test
    @DisplayName("Test HashMaps created from lines")
    void processLineTest() {
        String source = "unhinterfragt;unkritisch;bedenkenlos";
        Map<String, List<String>> target = new HashMap<>();
        target.put("unhinterfragt", List.of(new String[]{
                "unkritisch", "bedenkenlos"
        }));
        target.put("unkritisch", List.of(new String[]{
                "unhinterfragt", "bedenkenlos"
        }));
        target.put("bedenkenlos", List.of(new String[]{
                "unhinterfragt", "unkritisch"
        }));

        String source1 = "abhängen (von);(sich) richten (nach);(sich) bemessen (nach)";
        Map<String, List<String>> target1 = new HashMap<>();
        target1.put("abhängen", List.of(new String[]{
                "richten", "bemessen"
        }));
        target1.put("richten", List.of(new String[]{
                "abhängen", "bemessen"
        }));
        target1.put("bemessen", List.of(new String[]{
                "abhängen", "richten"
        }));

        assertTrue(twoMapsAreEqual(target, ThesaurusDictionary.processLine(source)));
        assertTrue(twoMapsAreEqual(target1, ThesaurusDictionary.processLine(source1)));
    }

    @Test
    @DisplayName("Created map should contain all words as keys")
    void createThesaurusDictionaryKeysTest () {
        String testData = String.join(newLine,
                "Kanopenkasten;Kanopentruhe;Eingeweidekasten",
                "Wiederaufnahme;Fortführung"
        );
        InputStream testDataInputStream = new ByteArrayInputStream(
                testData.getBytes(StandardCharsets.UTF_8)
        );
        List<String> keys = List.of(
                "Kanopenkasten", "Kanopentruhe", "Eingeweidekasten",
                "Wiederaufnahme", "Fortführung"
        );

        Map<String, List<List<String>>> testMap =
                ThesaurusDictionary.createThesaurusDictionary(testDataInputStream);

        for (String key : keys) {
            assertTrue(testMap.containsKey(key));
        }
    }

    @Test
    @DisplayName("Created map should merge same words and concatenate synsets")
    void createThesaurusDictionarySynsetsTest () {
        String testData = String.join(newLine,
                "gehen;laufen;rennen",
                "gehen;klappen;gelingen"
        );
        InputStream testDataInputStream = new ByteArrayInputStream(
                testData.getBytes(StandardCharsets.UTF_8)
        );
        List<List<String>> gehenSynsetsTarget = List.of(
                List.of("laufen", "rennen"),
                List.of("klappen", "gelingen")
        );

        Map<String, List<List<String>>> testMap =
                ThesaurusDictionary.createThesaurusDictionary(testDataInputStream);

        List<List<String>> gehenSynsetsResult = testMap.get("gehen");

        System.out.println(testMap);

        assertTrue(gehenSynsetsResult.containsAll(gehenSynsetsTarget));
    }
}
